@@ -1,5 +1,5 @@
 """
- * $Id: 1.0+16 utils.py 52dfe2315c83 2021-05-18 od $
+ * $Id: 1.1+2 utils.py 7df57bbd15f8 2021-05-19 od $
  *
  * This file is part of the Cloud Services Integration Platform (CSIP),
  * a Model-as-a-Service framework, API and application suite.
@@ -70,11 +70,16 @@ class Client(object):
     HDR_CSIP_WEBHOOK = "X-CSIP-Webhook"
     HDR_CSIP_REQUESTFILE = "X-CSIP-Request-File"
 
+    # CONFIG defaults
+    GET_TIMEOUT = (2,10) # (connect, read)
+    POST_TIMEOUT = (2,30) # (connect, read)
+    RETRY = 3
+    ALLOW_REDIRECTS = True
+
     def __init__(self, data: Dict = None, metainfo: Dict = None,
                  parent: 'Client' = None, url: str = None, name: str = '',
-                 descr: str = '', http_status: int = 0, conf: Dict = None):
+                 descr: str = '', http_status: int = 0):
         self.http_status = http_status
-        self.conf = conf or {}
         # parameter data or result data
         self.data = Client.__create_dict(data or [])
         # metainfo 
@@ -139,20 +144,20 @@ class Client(object):
 
         # data = {} if not section in response_json else response_json[section]
         # mi = {} if http_status != 200 else response_json[Client.METAINFO]
-        return Client(data, metainfo=mi, http_status=http_status, conf=conf)
+        return Client(data, metainfo=mi, http_status=http_status)
 
     @staticmethod
     def __get0(url: str, conf: Dict):
         """HTTP GET to generate a Client"""
         conf = conf or {}
-        to = conf.get('get_timeout', 4.0)
-        retry = conf.get('retry', 3)
-        allow_redirects = conf.get('allow_redirects', True)
+        timeout = conf.get('get_timeout', Client.GET_TIMEOUT)
+        retry = conf.get('retry', Client.RETRY)
+        allow_redirects = conf.get('allow_redirects', Client.ALLOW_REDIRECTS)
         http_status = 200
         response = None
         while retry > 0:
             try:
-                response = requests.get(url, allow_redirects=allow_redirects, timeout=to)
+                response = requests.get(url, allow_redirects=allow_redirects, timeout=timeout)
                 response.raise_for_status()
                 # response_json = response.json()
                 break
@@ -179,15 +184,15 @@ class Client(object):
     def __post0(url: str, files: Dict, headers: Dict, conf: Dict):
         """HTTP POST to generate a Client"""
         conf = conf or {}
-        to = conf.get('post_timeout', 10.0)
-        retry = conf.get('retry', 3)
-        allow_redirects = conf.get('allow_redirects', True)
+        timeout = conf.get('post_timeout', Client.POST_TIMEOUT)
+        retry = conf.get('retry', Client.RETRY)
+        allow_redirects = conf.get('allow_redirects', Client.ALLOW_REDIRECTS)
         http_status = 200
         response_json = None
         while retry > 0:
             try:
                 response = requests.post(url, files=files, allow_redirects=allow_redirects, headers=headers,
-                                         timeout=to)
+                                         timeout=timeout)
                 response.raise_for_status()
                 response_json = response.json()
                 break
@@ -242,7 +247,7 @@ class Client(object):
             raise Exception("HTTP Error " + http_status)
 
         response_json = response.json()
-        return [Client(metainfo=service, conf=conf) for service in response_json]
+        return [Client(metainfo=service) for service in response_json]
 
     @staticmethod
     def get_capabilities(c, conf: Dict = None) -> "Client":
@@ -272,7 +277,7 @@ class Client(object):
         with open(file, 'w') as outfile:
             json.dump(d, outfile)
 
-    def pubsub(self, url: str, callback=None, extra_metainfo: Dict = None) -> None:
+    def pubsub(self, url: str, callback=None, extra_metainfo: Dict = None, conf: Dict = None) -> None:
         """Publish/subscribe execution """
 
         def cb(client: 'Client', file: str, counter: int):
@@ -304,14 +309,15 @@ class Client(object):
                 c = Client.from_file(file)
                 if extra_metainfo is not None:
                     c.add_metainfo(extra_metainfo)
-                resp = c.execute(url, True, None, header)
+                resp = c.execute(url, True, None, header, conf)
                 callback(resp, file, i + 1)
                 if self._delay > 0:
                     sleep(self._delay / 1000)
             else:
                 break  # after max, we are done
 
-    def execute(self, url: str = None, sync: bool = True, files: List[str] = None, headers: Dict = None) -> 'Client':
+    def execute(self, url: str = None, sync: bool = True,
+                files: List[str] = None, headers: Dict = None, conf: Dict = None) -> 'Client':
         """Executes a service as HTTP/POST. This method returns when service is finished if sync
            argument is set to True (default). Creates a reponse client.
            Use execute_async to call a service asynchronously.
@@ -334,7 +340,7 @@ class Client(object):
         # print(multipart)
         # response = requests.post(url, files=multipart, allow_redirects=True, headers=headers)
 
-        response_json, http_status = Client.__post0(url, multipart, headers, self.conf)
+        response_json, http_status = Client.__post0(url, multipart, headers, conf)
 
         if http_status == 200:
             data = response_json.get(Client.RESULT, {})
@@ -346,11 +352,11 @@ class Client(object):
         # print(response_json)
         # print(http_status)
 
-        return Client(data, parent=self, metainfo=mi, http_status=http_status, conf=self.conf)
+        return Client(data, parent=self, metainfo=mi, http_status=http_status)
 
     def execute_async(self, url: str = None, files: List[str] = None,
                       first_poll: int = None, next_poll: int = None,
-                      callback=None, headers: Dict = None) -> 'Client':
+                      callback=None, headers: Dict = None, conf: Dict = None) -> 'Client':
         """Executes a service as HTTP/POST. This method runs the service asynchronously and
            returns when the service is done. It 'pings' the service run for completion in the
            frequency provided by 'fist_poll' and 'next_poll'. Provide a callback method to capture
@@ -373,7 +379,7 @@ class Client(object):
         callback = callback or cb
 
         # -> Submitted
-        resp = self.execute(url, files=files, sync=False, headers=headers)
+        resp = self.execute(url, files=files, sync=False, headers=headers, conf=conf)
         callback(resp, "")
 
         fp = (first_poll or resp.get_first_poll()) / 1000
@@ -388,19 +394,19 @@ class Client(object):
         sleep(fp)
 
         # Submitted -> Running
-        r = resp.query_execute()
+        r = resp.query_execute(conf=conf)
         progress = r.get_progress()
         callback(r, progress)
         while r.get_status() == Client.STATUS_RUNNING \
                 and r.get_http_status() == 200:
             sleep(np)
             # Running -> Running | Finished | Failed
-            r = resp.query_execute()
+            r = resp.query_execute(conf=conf)
             progress = r.get_progress()
             callback(r, progress)
         return r
 
-    def query_execute(self, suid: str = None) -> 'Client':
+    def query_execute(self, suid: str = None, conf: Dict = None) -> 'Client':
         """Queries the service execution for completion status. 
            The service must run in 'async' mode"""
 
@@ -420,7 +426,7 @@ class Client(object):
             raise Exception("no suid.")
         query_url = service[:a.start()] + "/q/" + suid
         # print(query_url)
-        c = Client.__get(query_url, section=self.RESULT, conf=self.conf)
+        c = Client.__get(query_url, section=self.RESULT, conf=conf)
         c.parent = self.parent
         return c
 
@@ -439,7 +445,7 @@ class Client(object):
                 files.append(name)
         return files
 
-    def download_data_files(self, files: List[str] = None, dir: str = ".") -> None:
+    def download_data_files(self, files: List[str] = None, dir: str = ".", conf: Dict = None) -> None:
         """ Downloads file(s) if this Client object is a valid response and a query url as value.
             If no 'file' argument is provided, all files will be dowloaded.
             If no 'dir' is provided, the current directory will be used.
@@ -461,8 +467,9 @@ class Client(object):
                 raise Exception("Not in data: " + name)
             val = self.get_data_value(name)
             print("  --> " + name)
-            response = Client.__get0(val, self.conf)
-            open(os.path.join(dir, name), "wb").write(response.content)
+            response = Client.__get0(val, conf)
+            with open(os.path.join(dir, name), 'wb') as outfile:
+                outfile.write(response.content)
 
     #### pubsub related properties
 
@@ -513,16 +520,6 @@ class Client(object):
         """Set the delay between p/s submissions in ms"""
         self._delay = d
 
-    @property
-    def config(self) -> Dict:
-        """ ??? """
-        return self.conf
-
-    @delay.setter
-    def config(self, c: Dict) -> None:
-        """ ??? """
-        self.conf = c
-
     #### metainfo
 
     def get_metainfo(self, name: str) -> str:
